#!/usr/bin/perl -w
#
# buildmenu.pl
# David Rowe 9 Nov 2006
#
# From asciidoc format text file extract top-level
# headings to use as menu options

# slurp text file ----------------------------------

my $textFile = shift;
{
    local( $/, *FH );
    open FH, $textFile or die "Can't open $textFile\n";
    $text = <FH>;
    close FH;
}

# to generate menu we need to find the tag associated with
# every level one heading.  As a first step we check that
# every level one heading has a tag - if it doesnt we display
# a list of those that don't and exit.  These need to be
# fixed manually before continuing.

@no_tag_headings = $text =~ /^\s*\n(\w.*)\n\-\-\-\-/mg;

$num_no_tags = @no_tag_headings;
if ($num_no_tags) {
    print "These headings have no tags, pls fix:\n";
    foreach (@no_tag_headings) {
	print "  $_\n";
    }
    exit;
}

# extract all level 1 headings into an array,
# pattern is line starting with whitespace or [, text line , line with _____
# extract all associated tags in a seperate array

@headings = $text =~ /^\[.*\n(\w.*)\n\-\-\-\-/mg;
@tags = $text =~ /^\[\[(.*)\]\]\n\w.*\n\-\-\-\-/mg;

# now generate html file, this is included into layout1.conf
# when asciidoc runs

$outFile = shift;

open FH, ">$outFile" or die "Can't open $outFile\n";
foreach (@headings) {
    print FH "<div><a href=\"#", shift @tags, "\">$_</a></div>\n";
}

