/*
  techo.c
  David Rowe 13 Sep 2006
 
  Unit test for zaptel echo canceller.  Used to assist port to and
  optimisation for the Blackfin.
*/

/*
  Copyright (C) 2006 David Rowe
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
*/

#include <stdio.h>
#include <stdlib.h>
#include "mec2.h"

#define TAPS    128    /* number of taps in echo canceller   */
#define SAMPLES 80000    /* number of sample to proces in test */
#define SAM_CYC 100    /* print cycles for this many samples */
#define ECHO    0.1    /* simple simulated echo model        */
#define AMP     32767  /* maximum amplitude of input sample  */

/* C-callable function to return value of CYCLES register */

unsigned int cycles() {
  int ret;

   __asm__ __volatile__ 
   (
   "%0 = CYCLES;\n\t"
   : "=&d" (ret)
   : 
   : "R1"
   );

   return ret;
}

int main() {
  echo_can_state_t *ec;
  int               i;
  short             txlin, rxlin, rxlin_ec;
  int               before;

  ec = echo_can_create(TAPS, 0);

  for(i=0; i<SAMPLES; i++) {
    txlin = AMP*((float)rand()/RAND_MAX);
    rxlin = ECHO*((float)txlin) + 0.5;
    before = cycles();
    rxlin_ec = echo_can_update(ec, txlin, rxlin);
    if (i<SAM_CYC)
      printf("cycles = %d\n", cycles() - before);
  }

  /* dump first few taps of echo model */

  for(i=0; i<10; i++) 
    printf("a_i[%d] = %f\n",i,ec->a_i[i]/pow(2,31));

  echo_can_free(ec);

  return 0;
}
