/*
  tsport.c
  David Rowe 22 June 2006
 
  Linux device driver for Blackfin that implements unit tests
  for the SPORT side of the Blackfin-Sport-SiLabs 3050 interface.  
  Tests out SPORT config in multichannel mode, using DMA and
  interrupts.

  Tested on Blackfin BF533 STAMP with an external 2.048MHz clock.

  See bfsi.c for more information.
*/

/*
  Copyright (C) 2006 David Rowe
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
*/

#include <linux/module.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/mm.h>
#include <bfsi.h>
/* 
   Need to do include bfsi module this way due to kbuild restrictions.
   I would rather have it as seperately compiled module but I
   couldn't work out how to specify the path I wanted
*/
#include "../../src/bfsi.c" 

#define SAMPLES 8

static char read_snapshot[SAMPLES];
static int isrs = 0;;

/* demo versions of call back functions */

void isr_callback(u8 *read_samples, u8 *write_samples) {
  memcpy(read_snapshot, read_samples, SAMPLES);
  isrs++;
}

int tsport_proc_read(char *buf, char **start, off_t offset, 
		     int count, int *eof, void *data)
{
	int  i;
	char buf1[80];
	
	sprintf(buf,"isrs.........: %d\n"
		    "read_snapshot:", 
		    isrs);

	for(i=0; i<SAMPLES; i++) {
	  sprintf(buf1, " 0x%02x", read_snapshot[i] & 0xff);
	  strcat(buf, buf1);
	}
	strcat(buf,"\n");

	*eof=1;
	return strlen(buf);
}

/*----------------------- MODULE INIT FUNCTION --------------------------*/

static int __init tsport_init_module (void) {

  printk("initializing module\n");
		
  bfsi_sport_init(isr_callback, SAMPLES, 1);
  create_proc_read_entry("tsport", 0, NULL, tsport_proc_read, NULL);

  return 0;
}

/*---------------------- MODULE CLEANUP FUNCTION -----------------------*/

static void __exit tsport_cleanup_module (void) {
  printk("cleaning up module\n");

  bfsi_sport_close();
  remove_proc_entry("tsport", NULL);
}

module_init(tsport_init_module);
module_exit(tsport_cleanup_module);

MODULE_AUTHOR("David Rowe");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("Blackfin/SiLabs SPORT Unit Test");

